@@ -13,16 +13,19 @@ path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
 refdata = os.path.join(path, 'refdata/')
 
 
-def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
+def flux2electron(diameter, obscure, delta_lambda_per_pixel, spaxel_xsize, spaxel_ysize,
+                  wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
     """
 
     Args:
-        spaxel_xsize:
-        spaxel_ysize:
-        wavearr:
+        diameter：diameter of the telescope in cm
+        obscure：obscure fraction of the telescope, value within [0, 1]
+        spaxel_xsize: spaxel size in dispersion-axis in arcsec
+        spaxel_ysize: spaxel size in spatial-axis in arcsec
+        wavearr: wavelength range in Angstrom
         fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
-        throughtputwave:
-        throughtputvalue:
+        throughtputwave: wavelength array of the throughtput curve
+        throughtputvalue: throughput fraction
         fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'
 
     Returns:
@@ -31,12 +34,12 @@ def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave,
 
     cc = 3.0e18   # speed of light, A/s
     spaxel_area = spaxel_xsize * spaxel_ysize    # arcsec^2
-    d = 200  # diameter of the telescope, in cm unit
-    obscure = 0.0  # effective central obscuration, no unit
+    d = diameter  # diameter of the telescope, in cm unit
+    obscure = obscure  # effective central obscuration, no unit
     telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
-    delta_lambda_per_pixel = 1.755555  # per pixel
+    # delta_lambda_per_pixel = 1.755555  # per pixel
     delta_hz_per_specpixel = delta_lambda_per_pixel / wavearr**2 * cc  # s^-1
-    specsampling = 1.0  # 2 pixels
+    specsampling = 1.0  # pixel
     planckh = 6.626e-27  # erg*s
     hv = planckh * cc / wavearr  # erg
     QE = 1
@@ -152,7 +155,8 @@ def get_nsky(wavearr):
     return fluxskypp
 
 
-def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
+def get_background(throughputfile, diameter, obscure, delta_lambda_per_pixel,
+                   zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
 
     """
     Consider zodiacal background and earth shine background in different levels.
@@ -171,7 +175,7 @@ def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_
     """
 
     # get IFS throughtput
-    throughtputwave, throughtputvalue = get_throughput()
+    throughtputwave, throughtputvalue = get_throughput(throughputfile)
 
     # get the zodia spectrum, convert to e/s/spaxel/spec-elements
     filename = os.path.join(refdata, 'csst', 'background', 'zodi_spec.csv')
@@ -179,7 +183,8 @@ def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_
     zodia_wave = cat['wave']
     zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
     zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
-    zodia_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+    zodia_counts_interp = flux2electron(diameter, obscure, delta_lambda_per_pixel,
+                                        spaxel_xsize, spaxel_ysize,
                                        wavearr, zodia_spec_interp,
                                        throughtputwave, throughtputvalue,
                                        fluxunit='erg/s/cm^2/A')
@@ -190,7 +195,8 @@ def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_
     earthshine_wave = cat['wave']
     earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
     earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
-    earthshine_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+    earthshine_counts_interp = flux2electron(diameter, obscure, delta_lambda_per_pixel,
+                                       spaxel_xsize, spaxel_ysize,
                                        wavearr, earthshine_spec_interp,
                                        throughtputwave, throughtputvalue,
                                        fluxunit='erg/s/cm^2/A')
@@ -214,13 +220,18 @@ class DetectorCountsRate(source.ModelCube):
             self, config
         )
 
-        throughtputwave, throughtputvalue = get_throughput()
 
         instru = config['configuration']
-        self.source_counts = flux2electron(instru['spaxel_xsize'], instru['spaxel_ysize'],
-                                           self.wavecube, self.fluxcube,
-                                           throughtputwave, throughtputvalue,
-                                           fluxunit='erg/s/cm^2/A')
+
+        throughtputwave, throughtputvalue = get_throughput(instru['efficiency_file'])
+
+        self.source_counts = flux2electron(instru['diameter'],
+                                               instru['obscure'],
+                                               instru['wave_delta'],
+                                               instru['spaxel_xsize'], instru['spaxel_ysize'],
+                                               self.wavecube, self.fluxcube,
+                                               throughtputwave, throughtputvalue,
+                                               fluxunit='erg/s/cm^2/A')
         self.wavearr = self.wavecube
 
 
@@ -236,7 +247,11 @@ class DetectorCounts(object):
         readnoisecube = np.zeros(shape=signal.source_counts.shape, dtype=float) + instru['readout_noise']   # e/pix
 
         # nskycube = get_nsky(signal.ccdspec_wave)        # e/s/specsampling
-        nskycube = get_background(config['zodiacal_level'],
+        nskycube = get_background(instru['efficiency_file'],
+                                  instru['diameter'],
+                                  instru['obscure'],
+                                  instru['wave_delta'],
+                                  config['zodiacal_level'],
                                    config['earthshine_level'],
                                    signal.ccdspec_wave,
                                    instru['spaxel_xsize'],
@@ -353,24 +368,25 @@ class calculation_snr(object):
         k = 0
         # i = points[k][0]
         # j = points[k][1]
-        ax_snr = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+
+        ax_recontructed_spec = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
+        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
+        ax_recontructed_spec.set_ylabel('mock spec')
+        ax_recontructed_spec.set_xlabel('wavelength')
+
+        ax_snr = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
         ax_snr.plot(self.mockwave, self.snr, 'k-')
         ax_snr.set_ylabel('SNR')
         ax_snr.set_xlabel('wavelength')
 
-        ax_noises = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
+        ax_noises = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
         ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
         ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
         ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
         ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
         ax_noises.legend(loc=1)
-        ax_noises.set_ylabel('different counts')
-
-        ax_recontructed_spec = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
-        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
-        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
-        ax_recontructed_spec.set_ylabel('mock spec')
-        ax_recontructed_spec.set_xlabel('wavelength')
+        ax_noises.set_ylabel('noise counts (e-)')
 
         plt.savefig(outfile)
         plt.close()
@@ -424,7 +440,7 @@ class calculation_limitmag(object):
 
         source_rate = (-coeff_s1 + np.sqrt(coeff_s1 ** 2 - 4 * coeff_s2 * coeff_s0)) / (2 * coeff_s2)
 
-        throughtputwave, throughtputvalue = get_throughput()
+        throughtputwave, throughtputvalue = get_throughput(instru['efficiency_file'])
         flux = electron2flux(config, ccdspec_wave, source_rate,
                              throughtputwave, throughtputvalue,
                              fluxunit='erg/s/cm^2/A')
@@ -449,7 +465,11 @@ class calculation_exptime(object):
         rn = instru['readout_noise']   # e/pix
         obst = config['obst']
         # nsky = get_nsky(ccdspec_wave)
-        nsky = get_background(config['zodiacal_level'],
+        nsky = get_background(instru['efficiency_file'],
+                              instru['diameter'],
+                              instru['obscure'],
+                              instru['wave_delta'],
+                              config['zodiacal_level'],
                                   config['earthshine_level'],
                                   ccdspec_wave,
                                   instru['spaxel_xsize'],
@@ -550,24 +570,25 @@ class calculation_exptime(object):
         k = 0
         # i = points[k][0]
         # j = points[k][1]
-        ax_snr = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+
+        ax_recontructed_spec = plt.subplot2grid((3, 3), (k, 0), colspan=3)
+        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
+        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
+        ax_recontructed_spec.set_ylabel('mock spec')
+        ax_recontructed_spec.set_xlabel('wavelength')
+
+        ax_snr = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
         ax_snr.plot(self.mockwave, self.snr, 'k-')
         ax_snr.set_ylabel('SNR')
         ax_snr.set_xlabel('wavelength')
 
-        ax_noises = plt.subplot2grid((3, 3), (k + 1, 0), colspan=3)
+        ax_noises = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
         ax_noises.plot(self.mockwave, self.sourcenoise, label='source_noise')
         ax_noises.plot(self.mockwave, self.skynoise, label='sky_noise')
         ax_noises.plot(self.mockwave, self.readnoise, label='readout_noise')
         ax_noises.plot(self.mockwave, self.darknoise, label='dark_noise')
         ax_noises.legend(loc=1)
-        ax_noises.set_ylabel('different counts')
-
-        ax_recontructed_spec = plt.subplot2grid((3, 3), (k + 2, 0), colspan=3)
-        ax_recontructed_spec.plot(self.mockwave, self.mockflux, 'k-', label='mockflux')
-        ax_recontructed_spec.plot(self.mockwave, self.mockerror, 'b-', label='mockerr')
-        ax_recontructed_spec.set_ylabel('mock spec')
-        ax_recontructed_spec.set_xlabel('wavelength')
+        ax_noises.set_ylabel('noise counts (e-)')
 
         plt.savefig(outfile)
         plt.close()
