@@ -139,6 +139,14 @@ class read_filter(object):
             self.wavemin = self.wave[0]
             self.wavemax = self.wave[-1]
 
+        elif filter0 == 'GALEX':
+            filter_file = os.path.join(refdata, 'normalization', 'filters', filter_name+'.par')
+            band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
+            self.wave = band[0].values             # A
+            self.throughput = band[1].values       # no sure per photon, or per energy. (??)
+            self.wavemin = self.wave[0]
+            self.wavemax = self.wave[-1]
+
         else:
             print('un_recognized filter system.')
 
