@@ -6,63 +6,62 @@ import h5py
 from einops import repeat
 import matplotlib.pyplot as plt
 from . import source
-from .source import filter_mag, read_filter
-from .config import get_throughput
+from ..etc1d.perform_calculation import flux2electron, get_background, get_throughput
 
 path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
 snpp_refdata = os.path.join(path, 'refdata/')
 
-def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
-    """
-
-    Args:
-        spaxel_xsize: 
-        spaxel_ysize: 
-        wavearr: 
-        fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
-        throughtputwave: 
-        throughtputvalue: 
-        fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'
-
-    Returns:
-
-    """
-
-    # instru = config['configuration']
-
-    cc = 3.0e18   # speed of light, A/s
-    spaxel_area = spaxel_xsize * spaxel_ysize     # arcsec^2
-    d = 200  # diameter of the telescope, in cm unit
-    obscure = 0.0  # effective central obscuration, no unit
-    telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
-    delta_lambda_per_pixel = 1.755555  # per pixel
-    delta_hz_per_specpixel = delta_lambda_per_pixel / wavearr**2 * cc  # s^-1
-    specsampling = 1.0  # 2 pixels
-    planckh = 6.626e-27  # erg*s
-    hv = planckh * cc / wavearr  # erg
-    QE = 1
-
-    if wavearr.ndim == 3:
-        qvalue = np.zeros(shape=wavearr.shape, dtype=float)
-        qvalue = np.interp(wavearr[:, 0, 0], throughtputwave, throughtputvalue)
-        qvalue = repeat(qvalue, 'h -> h '+str(wavearr.shape[1])+' '+str(wavearr.shape[2]))
-    elif wavearr.ndim == 1:
-        qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)
-    else:
-        raise ValueError('check dimension of the input array.')
-
-    if fluxunit=='erg/s/cm^2/A':
-        isource1 = fluxarr * wavearr ** 2 / cc * spaxel_area  # erg/s/cm^2/Hz per spaxel
-        isource2 = isource1 * telarea  # erg/s/Hz per spaxel
-        isource3 = qvalue * isource2 * delta_hz_per_specpixel  # erg/s per spaxel
-        isource4 = isource3 * specsampling  # erg/s per spec-element per spaxel
-        isource5 = isource4 / hv  # phot/s per spec-element per spaxel
-        isource6 = isource5 * QE  # e/s per spec-element per spaxel
-
-        return isource6
-    else:
-
-        return np.nan
+# def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
+#     """
+#
+#     Args:
+#         spaxel_xsize:
+#         spaxel_ysize:
+#         wavearr:
+#         fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
+#         throughtputwave:
+#         throughtputvalue:
+#         fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'
+#
+#     Returns:
+#
+#     """
+#
+#     # instru = config['configuration']
+#
+#     cc = 3.0e18   # speed of light, A/s
+#     spaxel_area = spaxel_xsize * spaxel_ysize     # arcsec^2
+#     d = 200  # diameter of the telescope, in cm unit
+#     obscure = 0.0  # effective central obscuration, no unit
+#     telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
+#     delta_lambda_per_pixel = 1.755555  # per pixel
+#     delta_hz_per_specpixel = delta_lambda_per_pixel / wavearr**2 * cc  # s^-1
+#     specsampling = 1.0  # 2 pixels
+#     planckh = 6.626e-27  # erg*s
+#     hv = planckh * cc / wavearr  # erg
+#     QE = 1
+#
+#     if wavearr.ndim == 3:
+#         qvalue = np.zeros(shape=wavearr.shape, dtype=float)
+#         qvalue = np.interp(wavearr[:, 0, 0], throughtputwave, throughtputvalue)
+#         qvalue = repeat(qvalue, 'h -> h '+str(wavearr.shape[1])+' '+str(wavearr.shape[2]))
+#     elif wavearr.ndim == 1:
+#         qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)
+#     else:
+#         raise ValueError('check dimension of the input array.')
+#
+#     if fluxunit == 'erg/s/cm^2/A':
+#         isource1 = fluxarr * wavearr ** 2 / cc * spaxel_area  # erg/s/cm^2/Hz per spaxel
+#         isource2 = isource1 * telarea  # erg/s/Hz per spaxel
+#         isource3 = qvalue * isource2 * delta_hz_per_specpixel  # erg/s per spaxel
+#         isource4 = isource3 * specsampling  # erg/s per spec-element per spaxel
+#         isource5 = isource4 / hv  # phot/s per spec-element per spaxel
+#         isource6 = isource5 * QE  # e/s per spec-element per spaxel
+#
+#         return isource6
+#     else:
+#
+#         return np.nan
 
 
 def electron2flux(config, wavearr, countsarr, throughtputwave, throughtputvalue, fluxunit=''):
@@ -152,53 +151,53 @@ def get_ms_nsky(wavearr, spaxel_xsize, spaxel_ysize):
 
 
 
-def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
-
-    """
-    Consider zodiacal background and earth shine background in different levels.
-
-    Args:
-        zodia_level:
-            zodiacal level, low, median, high, corresponding index from 1-3.
-        earthshine_level:
-            earth shine level, low, median, high, corresponding index from 1-3.
-        wavearr:
-            wave array to be returned.
-
-    Returns:
-        sky counts (e-/s) in each spaxel element (spaxel_xsize * spaxel_ysize), in 1d array.
-
-    """
-
-    # get IFS throughtput
-    throughtputwave, throughtputvalue = get_throughput()
-
-    # get the zodia spectrum, convert to e/s/spaxel/spec-elements
-    filename = os.path.join(snpp_refdata, 'csst', 'background', 'zodi_spec.csv')
-    cat = pd.read_csv(filename, comment='#')
-    zodia_wave = cat['wave']
-    zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
-    zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
-    zodia_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
-                                       wavearr, zodia_spec_interp,
-                                       throughtputwave, throughtputvalue,
-                                       fluxunit='erg/s/cm^2/A')
-
-    # get the earth shine spectrum, convert to e/s/spaxel/spec-elements
-    filename = os.path.join(snpp_refdata, 'csst', 'background', 'earthshine_spec.csv')
-    cat = pd.read_csv(filename, comment='#')
-    earthshine_wave = cat['wave']
-    earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
-    earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
-    earthshine_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
-                                       wavearr, earthshine_spec_interp,
-                                       throughtputwave, throughtputvalue,
-                                       fluxunit='erg/s/cm^2/A')
-
-    nsky = zodia_counts_interp + earthshine_counts_interp
-
-
-    return nsky
+# def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
+#
+#     """
+#     Consider zodiacal background and earth shine background in different levels.
+#
+#     Args:
+#         zodia_level:
+#             zodiacal level, low, median, high, corresponding index from 1-3.
+#         earthshine_level:
+#             earth shine level, low, median, high, corresponding index from 1-3.
+#         wavearr:
+#             wave array to be returned.
+#
+#     Returns:
+#         sky counts (e-/s) in each spaxel element (spaxel_xsize * spaxel_ysize), in 1d array.
+#
+#     """
+#
+#     # get IFS throughtput
+#     throughtputwave, throughtputvalue = get_throughput()
+#
+#     # get the zodia spectrum, convert to e/s/spaxel/spec-elements
+#     filename = os.path.join(snpp_refdata, 'csst', 'background', 'zodi_spec.csv')
+#     cat = pd.read_csv(filename, comment='#')
+#     zodia_wave = cat['wave']
+#     zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
+#     zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
+#     zodia_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+#                                        wavearr, zodia_spec_interp,
+#                                        throughtputwave, throughtputvalue,
+#                                        fluxunit='erg/s/cm^2/A')
+#
+#     # get the earth shine spectrum, convert to e/s/spaxel/spec-elements
+#     filename = os.path.join(snpp_refdata, 'csst', 'background', 'earthshine_spec.csv')
+#     cat = pd.read_csv(filename, comment='#')
+#     earthshine_wave = cat['wave']
+#     earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
+#     earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
+#     earthshine_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+#                                        wavearr, earthshine_spec_interp,
+#                                        throughtputwave, throughtputvalue,
+#                                        fluxunit='erg/s/cm^2/A')
+#
+#     nsky = zodia_counts_interp + earthshine_counts_interp
+#
+#
+#     return nsky
 
 
 
@@ -208,16 +207,17 @@ class SourceCountsRate(source.ModelCube):
 
         self.config = config
 
-        # pdb.set_trace()
-
-        source.ModelCube.__init__(
-            self, config
-        )
-
-        throughtputwave, throughtputvalue = get_throughput()
+        source.ModelCube.__init__(self, config)    # 按config参数生成fluxcube, wavecube
+                                                   # 上传的cube也在这一步插值成符合ccdgrid的cube
 
         instru = config['configuration']
-        self.source_counts = flux2electron(instru['spaxel_xsize'], instru['spaxel_ysize'],
+
+        throughtputwave, throughtputvalue = get_throughput(instru['efficiency_file'])
+
+        self.source_counts = flux2electron(instru['diameter'],
+                                           instru['obscure'],
+                                           instru['wave_delta'],
+                                           instru['spaxel_xsize'], instru['spaxel_ysize'],
                                            self.wavecube, self.fluxcube,
                                            throughtputwave, throughtputvalue,
                                            fluxunit='erg/s/cm^2/A')
@@ -242,7 +242,11 @@ class NoiseCounts(object):
 
         nreadout_rate = np.zeros(shape=nw, dtype=float) + instru['readout_noise']**2    # e/pix
 
-        nsky_rate = get_background(config['background']['zodiacal'],
+        nsky_rate = get_background(instru['efficiency_file'],
+                                   instru['diameter'],
+                                   instru['obscure'],
+                                   instru['wave_delta'],
+                                   config['background']['zodiacal'],
                                    config['background']['earth_shine'],
                                    signal.ccdspec_wave,
                                    instru['spaxel_xsize'],
@@ -372,9 +376,16 @@ class calculation_snr(object):
         # model image
         self.img2d = signal_rate.mag2d
 
+        import ifs_etc
+        self.__version__ = ifs_etc.__version__
+
     def save(self, file):
 
+        import ifs_etc
+        version = ifs_etc.__version__
+
         f = h5py.File(file, 'w')
+        f.create_dataset('__version__', data=version)
         f.create_dataset('source', data=self.source)
         f.create_dataset('darknoise', data=self.darknoise)
         f.create_dataset('readnoise', data=self.readnoise)
@@ -389,6 +400,7 @@ class calculation_snr(object):
         f.create_dataset('mockflux', data=self.mockflux)
         f.create_dataset('mockerror', data=self.mockerror)
         f.create_dataset('img2d', data=self.img2d)
+
         f.close()
 
 
@@ -431,7 +443,7 @@ class calculation_snr(object):
         ax_snr = plt.subplot2grid((3, 3), (k, 1), colspan=2)
         ax_snr.plot(self.mockwave, self.snr[:, j, i], 'k-')
         ax_snr.annotate('surface brightness: \n' + '{:4.1f}'.format(self.img2d[j, i]) + r' mag/arcsec$^{2}$',
-                        xy=(0.75, 0.8), xycoords='axes fraction')
+                        xy=(0.75, 0.8), xycoords='axes fraction', color='red')
         ax_snr.annotate('SNR@5000A: \n' + '{:4.1f}'.format(self.snr[ind, j, i]),
                         xy=(0.75, 0.65), xycoords='axes fraction')
         ax_snr.annotate('x: ' + str(i) + ', y: ' + str(j),
@@ -474,7 +486,7 @@ class calculation_snr(object):
             ax_snr = plt.subplot2grid((3, 3), (k, 1), colspan=2)
             ax_snr.plot(self.mockwave, self.snr[:, j, i], 'k-')
             ax_snr.annotate('surface brightness: \n' + '{:4.1f}'.format(self.img2d[j, i]) + r' mag/arcsec$^{2}$',
-                            xy=(0.75, 0.8), xycoords='axes fraction')
+                            xy=(0.75, 0.8), xycoords='axes fraction', color='red')
             ax_snr.annotate('SNR@5000A: \n' + '{:4.1f}'.format(self.snr[ind, j, i]),
                             xy=(0.75, 0.65), xycoords='axes fraction')
             ax_snr.annotate('x: ' + str(i) + ', y: ' + str(j),
